@@ -7,7 +7,6 @@
 
 #if __GLASGOW_HASKELL__ >= 701
 {-# LANGUAGE DefaultSignatures #-}
-{-# LANGUAGE Safe #-}
 #endif
 
 #if __GLASGOW_HASKELL__ >= 705
@@ -18,6 +17,12 @@
 {-# LANGUAGE EmptyCase #-}
 #endif
 
+#if __GLASGOW_HASKELL__ >= 710
+{-# LANGUAGE Safe #-}
+#elif __GLASGOW_HASKELL__ >= 701
+{-# LANGUAGE Trustworthy #-}
+#endif
+
 module Generics.Deriving.Functor (
   -- * Generic Functor class
     GFunctor(..)
@@ -41,6 +46,12 @@ import           Generics.Deriving.Base
 import           Data.Complex (Complex)
 #endif
 
+#if MIN_VERSION_base(4,6,0)
+import           Data.Ord (Down)
+#else
+import           GHC.Exts (Down)
+#endif
+
 #if MIN_VERSION_base(4,7,0)
 import           Data.Proxy (Proxy)
 #endif
@@ -156,6 +167,9 @@ instance GFunctor Complex where
 instance GFunctor (Const m) where
   gmap = gmapdefault
 
+instance GFunctor Down where
+  gmap = gmapdefault
+
 instance GFunctor Dual where
   gmap = gmapdefault
 
