@@ -5,13 +5,18 @@
 
 #if __GLASGOW_HASKELL__ >= 701
 {-# LANGUAGE DefaultSignatures #-}
-{-# LANGUAGE Safe #-}
 #endif
 
 #if __GLASGOW_HASKELL__ >= 705
 {-# LANGUAGE PolyKinds #-}
 #endif
 
+#if __GLASGOW_HASKELL__ >= 710
+{-# LANGUAGE Safe #-}
+#elif __GLASGOW_HASKELL__ >= 701
+{-# LANGUAGE Trustworthy #-}
+#endif
+
 {- | This module provides two main features:
 
     1. 'GMonoid', a generic version of the 'Monoid' type class, including instances
@@ -71,6 +76,12 @@ import Control.Applicative
 import Data.Monoid
 import Generics.Deriving.Base
 
+#if MIN_VERSION_base(4,6,0)
+import Data.Ord (Down)
+#else
+import GHC.Exts (Down)
+#endif
+
 #if MIN_VERSION_base(4,7,0)
 import Data.Proxy (Proxy)
 #endif
@@ -219,6 +230,9 @@ instance GMonoid b => GMonoid (a -> b) where
 instance GMonoid a => GMonoid (Const a b) where
   gmempty  = gmemptydefault
   gmappend = gmappenddefault
+instance GMonoid a => GMonoid (Down a) where
+  gmempty  = gmemptydefault
+  gmappend = gmappenddefault
 
 #if MIN_VERSION_base(4,7,0)
 instance GMonoid
