@@ -30,11 +30,6 @@ import           Language.Haskell.TH.Lib
 import           Language.Haskell.TH.Ppr (pprint)
 import           Language.Haskell.TH.Syntax
 
-#if !(MIN_VERSION_base(4,8,0))
-import           Data.Foldable (Foldable(foldMap))
-import           Data.Monoid (Monoid(..))
-#endif
-
 #ifndef CURRENT_PACKAGE_KEY
 import           Data.Version (showVersion)
 import           Paths_generic_deriving (version)
@@ -104,45 +99,6 @@ hasKindStar (SigT _ StarK) = True
 #endif
 hasKindStar _              = False
 
--- | Gets all of the required type/kind variable binders mentioned in a Type.
--- This does not add separate items for kind variable binders (in contrast with
--- the behavior of 'freeVariables').
-requiredTyVarsOfTypes :: [Type] -> [TyVarBndr]
-requiredTyVarsOfTypes tys =
-  let fvs :: [Name]
-      fvs = nub $ concatMap freeVariables tys
-
-      varKindSigs :: Map Name Kind
-      varKindSigs = foldMap go tys
-        where
-          go :: Type -> Map Name Kind
-          go (ForallT {}) = error "`forall` type used in type family pattern"
-          go (AppT t1 t2) = go t1 `mappend` go t2
-          go (SigT t k) =
-            let kSigs =
-#if MIN_VERSION_template_haskell(2,8,0)
-                  go k
-#else
-                  mempty
-#endif
-            in case t of
-                 VarT n -> Map.insert n k kSigs
-                 _      -> go t `mappend` kSigs
-          go _ = mempty
-
-      ascribeWithKind n
-        | Just k <- Map.lookup n varKindSigs
-        = KindedTV n k
-        | otherwise
-        = PlainTV n
-
-      isKindBinder = (`Set.member` kindVars)
-        where
-          kindVars = Set.fromList $ concatMap freeVariables $ Map.elems varKindSigs
-
-  in map ascribeWithKind $
-     filter (not . isKindBinder) fvs
-
 -- | Converts a VarT or a SigT into Just the corresponding TyVarBndr.
 -- Converts other Types to Nothing.
 typeToTyVarBndr :: Type -> Maybe TyVarBndr
@@ -379,11 +335,11 @@ trd3 (_, _, c) = c
 shrink :: (a, b, c) -> (b, c)
 shrink (_, b, c) = (b, c)
 
--- | Variant of foldr1 which returns a special element for empty lists
-foldr1' :: (a -> a -> a) -> a -> [a] -> a
-foldr1' _ x [] = x
-foldr1' _ _ [x] = x
-foldr1' f x (h:t) = f h (foldr1' f x t)
+foldBal :: (a -> a -> a) -> a -> [a] -> a
+foldBal _  x []  = x
+foldBal _  _ [y] = y
+foldBal op x l   = let (a,b) = splitAt (length l `div` 2) l
+                   in foldBal op x a `op` foldBal op x b
 
 isNewtypeVariant :: DatatypeVariant_ -> Bool
 isNewtypeVariant Datatype_             = False
@@ -391,22 +347,6 @@ isNewtypeVariant Newtype_              = True
 isNewtypeVariant (DataInstance_ {})    = False
 isNewtypeVariant (NewtypeInstance_ {}) = True
 
-#if MIN_VERSION_template_haskell(2,7,0)
--- | Extracts the constructors of a data or newtype declaration.
-dataDecCons :: Dec -> [Con]
-dataDecCons (DataInstD _ _ _
-# if MIN_VERSION_template_haskell(2,11,0)
-                       _
-# endif
-                       cons _) = cons
-dataDecCons (NewtypeInstD _ _ _
-# if MIN_VERSION_template_haskell(2,11,0)
-                          _
-# endif
-                          con _) = [con]
-dataDecCons _ = error "Must be a data or newtype declaration."
-#endif
-
 -- | Indicates whether Generic or Generic1 is being derived.
 data GenericClass = Generic | Generic1 deriving Enum
 
@@ -424,8 +364,8 @@ data GenericKind = Gen0
 genericKind :: GenericClass -> [Type] -> ([TyVarBndr], GenericKind)
 genericKind gClass tySynVars =
   case gClass of
-    Generic  -> (requiredTyVarsOfTypes tySynVars, Gen0)
-    Generic1 -> (requiredTyVarsOfTypes initArgs, Gen1 (varTToName lastArg) mbLastArgKindName)
+    Generic  -> (freeVariablesWellScoped tySynVars, Gen0)
+    Generic1 -> (freeVariablesWellScoped initArgs, Gen1 (varTToName lastArg) mbLastArgKindName)
   where
     -- Everything below is only used for Generic1.
     initArgs :: [Type]
@@ -519,11 +459,15 @@ reifyDataInfo :: Name
 reifyDataInfo name = do
   return $ Left $ ns ++ " Could not reify " ++ nameBase name
  `recover`
-  do DatatypeInfo { datatypeContext = ctxt
-                  , datatypeName    = parentName
-                  , datatypeVars    = tys
-                  , datatypeVariant = variant
-                  , datatypeCons    = cons
+  do DatatypeInfo { datatypeContext   = ctxt
+                  , datatypeName      = parentName
+#if MIN_VERSION_th_abstraction(0,3,0)
+                  , datatypeInstTypes = tys
+#else
+                  , datatypeVars      = tys
+#endif
+                  , datatypeVariant   = variant
+                  , datatypeCons      = cons
                   } <- reifyDatatype name
      let variant_ = case variant of
                       Datatype        -> Datatype_
